diff -urN pdns-recursor-3.1.7.orig/iputils.hh pdns-recursor-3.1.7/iputils.hh
--- pdns-recursor-3.1.7.orig/iputils.hh	2008-06-25 05:23:33.000000000 +1100
+++ pdns-recursor-3.1.7/iputils.hh	2008-12-09 15:51:09.579438929 +1000
@@ -32,6 +32,7 @@
 #include <functional>
 #include "ahuexception.hh"
 #include "misc.hh"
+#include "dns_random.hh"
 #include <boost/tuple/tuple.hpp>
 #include <boost/tuple/tuple_comparison.hpp>
 #include <boost/lexical_cast.hpp>
@@ -151,6 +152,15 @@
     memcpy(&ret.sin4.sin_addr.s_addr, ptr, 4);
     return ret;
   }
+  
+  ComboAddress getIPv4 () 
+  {
+	ComboAddress ret;
+	ret.sin4.sin_family = AF_INET;
+	ret.sin4.sin_port = 0;
+	ret.sin4.sin_addr.s_addr = sin4.sin_addr.s_addr;
+	return ret;
+  }
 
   string toString() const
   {
@@ -326,4 +336,75 @@
   
 };
 
+enum {POLICY_ROUNDROBIN, POLICY_RANDOM, POLICY_LEASTLOAD };
+
+class ComboAddressGroup 
+{
+public:
+
+    ComboAddressGroup (int pol = POLICY_ROUNDROBIN)
+    {
+	this->policy = pol;
+    }
+    
+    void addAddr (const string &addr)
+    {
+	d_addrs.push_back(ComboAddress(addr));
+    }
+    
+    bool empty ()
+    {
+	return d_addrs.empty ();
+    }
+    
+    unsigned int size ()
+    {
+	return (unsigned int)d_addrs.size ();
+    }
+    
+    string toString () const
+    {
+    ostringstream str;
+    
+    for(container_t::const_iterator iter = d_addrs.begin(); iter != d_addrs.end(); ++iter) {
+      if(iter != d_addrs.begin())
+	str <<", ";
+      str<<iter->toString();
+    }
+    return str.str();
+    }
+    
+    ComboAddress get ()
+    {
+	container_t::iterator ent = d_addrs.begin ();
+	
+	if (size () == 0)
+	    return ComboAddress ();
+	else if (size () > 1)
+	{
+	    switch (policy)
+	    {
+		case POLICY_ROUNDROBIN:
+		    if (idx < 0 || idx >= size ())
+			idx = 0;
+		    break;
+		case POLICY_RANDOM:
+		    idx = dns_random (size ());
+		    break;
+		default:
+		    idx = 0;
+	    }
+	    ent += idx++;
+	}
+	return ent->getIPv4 ();
+    }
+    
+private:
+    typedef vector<ComboAddress> container_t;
+    container_t d_addrs;
+    
+    unsigned int idx;
+    unsigned int policy;
+};
+
 #endif
diff -urN pdns-recursor-3.1.7.orig/pdns_recursor.cc pdns-recursor-3.1.7/pdns_recursor.cc
--- pdns-recursor-3.1.7.orig/pdns_recursor.cc	2008-06-25 05:23:33.000000000 +1100
+++ pdns-recursor-3.1.7/pdns_recursor.cc	2008-12-09 15:47:29.074818305 +1000
@@ -80,6 +80,8 @@
 bool g_quiet;
 NetmaskGroup* g_allowFrom;
 NetmaskGroup* g_dontQuery;
+ComboAddressGroup *g_queryaddr;
+
 string s_programname="pdns_recursor";
 typedef vector<int> g_tcpListenSockets_t;
 g_tcpListenSockets_t g_tcpListenSockets;
@@ -213,6 +215,8 @@
     if(connect(*fd, (struct sockaddr*)(&toaddr), toaddr.getSocklen()) < 0) {
       int err = errno;
       //      returnSocket(*fd);
+      
+// Au. TODO. Set Hard error for g_gueryaddr
       Utility::closesocket(*fd);
       if(err==ENETUNREACH) // Seth "My Interfaces Are Like A Yo Yo" Arnold special
 	return -2;
@@ -250,25 +254,29 @@
     d_socks.erase(i++);
     --d_numsocks;
   }
-
+  
+// Au. Use g_queryaddr 
   // returns -1 for errors which might go away, throws for ones that won't
   int makeClientSocket(int family)
   {
     int ret=(int)socket(family, SOCK_DGRAM, 0);
+    optional <ComboAddress> sin4;
+    
     if(ret < 0 && errno==EMFILE) // this is not a catastrophic error
       return ret;
     
     if(ret<0) 
       throw AhuException("Making a socket for resolver: "+stringerror());
     
-    static optional<ComboAddress> sin4;
-    if(!sin4) {
-      sin4=ComboAddress(::arg()["query-local-address"]);
-    }
+    if (!g_queryaddr || g_queryaddr->size () == 0)
+	sin4 = ComboAddress ();
+    else
+	sin4 = g_queryaddr->get ();
+
     static optional<ComboAddress> sin6;
     if(!sin6) {
       if(!::arg()["query-local-address6"].empty())
-	sin6=ComboAddress(::arg()["query-local-address6"]);
+	sin6=ComboAddress(::arg()["query-local-address6"]); // What's about empty args ?! sigfault, I think
     }
     
     int tries=10;
@@ -292,7 +300,8 @@
     }
     if(!tries)
       throw AhuException("Resolver binding to local query client socket: "+stringerror());
-    
+      
+//    L<<Logger::Warning <<"Use "<<sin4->toString()<<" as query source address"<<endl;
     Utility::setNonBlocking(ret);
     return ret;
   }
@@ -336,6 +345,8 @@
   
   g_fdm->addReadFD(*fd, handleUDPServerResponse, pident);
   ret=send(*fd, data, len, 0);
+  
+//Au. TODO
   if(ret < 0)
     g_udpclientsocks.returnSocket(*fd);
   return ret;
@@ -1739,6 +1750,34 @@
     }
     L<<Logger::Warning<<endl;
   }
+  
+// Au. init g_queryaddr 
+  if (!::arg()["query-local-address"].empty ())
+  {
+    string s = ::arg()["query-address-policy"];
+    
+    unsigned int policy = POLICY_ROUNDROBIN;
+    
+    if (s == "random")
+	policy = POLICY_RANDOM;
+    else if (s == "least-loaded")
+	policy = POLICY_LEASTLOAD;
+    else // if (s == "round-robin")
+	policy = POLICY_ROUNDROBIN;
+    
+    g_queryaddr = new ComboAddressGroup (policy);
+    vector<string> ips;
+    
+    stringtok(ips, ::arg()["query-local-address"], ", ");
+    L<<Logger::Warning<<"Send queries from: ";
+    for(vector<string>::const_iterator i = ips.begin(); i!= ips.end(); ++i) {
+      g_queryaddr->addAddr(*i);
+      if(i!=ips.begin())
+	L<<Logger::Warning<<", ";
+      L<<Logger::Warning<<*i;
+    }
+    L<<Logger::Warning<<endl;
+  }
 
   g_quiet=::arg().mustDo("quiet");
   if(::arg().mustDo("trace")) {
@@ -1860,6 +1899,7 @@
 	TCPConnection conn=any_cast<TCPConnection>(i->second);
 	if(g_logCommonErrors)
 	  L<<Logger::Warning<<"Timeout from remote TCP client "<< conn.remote.toString() <<endl;
+//Au. TODO. timeout error 
 	g_fdm->removeReadFD(i->first);
 	conn.closeAndCleanup();
       }
@@ -1990,6 +2030,7 @@
     ::arg().set("delegation-only","Which domains we only accept delegations from")="";
     ::arg().set("query-local-address","Source IP address for sending queries")="0.0.0.0";
     ::arg().set("query-local-address6","Source IPv6 address for sending queries")="";
+    ::arg().set("query-address-policy", "query-local-address selection (random|round-robin|least-loaded)")="round-robin";
     ::arg().set("client-tcp-timeout","Timeout in seconds when talking to TCP clients")="2";
     ::arg().set("max-tcp-clients","Maximum number of simultaneous TCP clients")="128";
     ::arg().set("hint-file", "If set, load root hints from this file")="";
